(function(angular, $, _) {
	
	angular.module('form_processor').directive('crmFormProcessorSpecification', function($compile) {
	  return {
	    restrict: 'E',
	    transclude: true,
	    scope: {
	      specification: '=specification',
	      fields: '=fields',
	      configuration: '=configuration',
	      noLabel: '@',
	    },
	    templateUrl: '~/form_processor/config/crmFormProcessorSpecification.html',
	    // Code below is from: https://stackoverflow.com/a/19172067/3853493
	    // And is taken from that site because we need recursion of this directive.
	    compile: function(tElement, tAttr, transclude) {
	    	//We are removing the contents/innerHTML from the element we are going to be applying the 
        //directive to and saving it to adding it below to the $compile call as the template
        var contents = tElement.contents().remove();
        var compiledContents;
        return function(scope, iElement, iAttr) {
        	scope.ts = CRM.ts(null);
        	
        	scope.uiField = {
        		name: scope.specification.name, 
        	};
        	if (!scope.noLabel) {
        		scope.uiField['title'] = scope.specification.title;
        		scope.uiField['required'] = scope.specification.required;
        	}

        	scope.addItemToCollection = function addItemToCollection(specificationBag) {
        		var item_config = angular.copy(specificationBag.default_configuration);
        		scope.configuration[scope.specification.name].push(item_config);
        	};
        	
        	scope.removeItem = function removeItem(item) {
        		var index = scope.configuration[scope.specification.name].indexOf(item);
    				if (index >= 0) {
    					scope.configuration[scope.specification.name].splice(index, 1);
    				}
        	};
        	
        	
        	if(!compiledContents) {
        		//Get the link function with the contents frome top level template with 
        		//the transclude
						compiledContents = $compile(contents, transclude);
					}
          //Call the link function to link the given scope and
          //a Clone Attach Function, http://docs.angularjs.org/api/ng.$compile :
          // "Calling the linking function returns the element of the template. 
          //    It is either the original element passed in, 
          //    or the clone of the element if the cloneAttachFn is provided."
          compiledContents(scope, function(clone, scope) {
          	//Appending the cloned template to the instance element, "iElement", 
            //on which the directive is to used.
            iElement.append(clone); 
          });
        };
      }
	  };
	});
	
})(angular, CRM.$, CRM._);